/*
Plugin Name: Fit Image Jquery Plguin
Author: Angelo Nunez
Date Created: 01/24/2018
*/

$(function($) {
    var pluginName = 'fitImage',
        defaults = {
            containerSelector: 'div',
            verticalAlign: 'middle',
            horizontalAlign: 'center',
            resizeDelayTime: '250'
        };

    function Plugin(element, options) {
        this.element = element;
        this.settings = $.extend({}, defaults, options);
        this._defaults = defaults;
        this._name = pluginName;
        this.init();
    }
    $.extend(Plugin.prototype, {
        
        init: function() {
            this.setWrapperContainer(this.element);
        },
        
        setWrapperContainer: function($img) {
            var $parent = $(this.element).parent(this.settings.containerSelector);
            var style = "width:100%;height:100%;overflow:hidden;padding:0 !important;margin:0 !important;"
            var htmlString = "<div class='ss-wrap-image' data-vertical-align='" + this.settings.verticalAlign + "' data-horizontal-align='" + this.settings.horizontalAlign + "'>";
                htmlString += "<img class='ss-image' src='" + $($img).attr('src') + "'/>";
                htmlString += "</div>";
            var wrap = $(htmlString).attr('style', style);

            this.settings.img = $(wrap).find("img");
            if ($parent !== undefined && $parent.length !== 0) $($img).remove();
            
            this.setWidth(wrap);
            $parent.append(wrap);
            this.fitImage(wrap);
        },
        
        fitImageOnResize: function(wrap) {
            
            var $this = this;
            $(wrap).each(function(i) {
                
                $this.clearImageProps($(this));
                $this.setWidth($(this));
                
                var img = $this.getImageSize(this);
                var container = $this.getContainerSize(this);
                var image = $(this).find("img");
                
                if (container.height > img.height) {
                    
                    $(image).css({
                        'height': '100%',
                        'width': 'auto'
                    });
                    
                    img = $this.getImageSize(this);
                    var overlapWidth, marginLeft;
                    switch ($this.settings.horizontalAlign.toLowerCase()) {
                        case 'center':
                            {
                                overlapWidth = container.width - img.width;
                                marginLeft = overlapWidth / 2;
                                $(image).css({
                                    'margin-left': marginLeft + 'px'
                                });
                                break;
                            }
                        case 'right':
                            {
                                overlapWidth = container.width - img.width;
                                marginLeft = overlapWidth;
                                $(image).css({
                                    'margin-left': marginLeft + 'px'
                                });
                                break;
                            }
                        default:
                            break;
                    }
                } else {
                    
                    img = $this.getImageSize(this);
                    var overlapHeight, marginTop;
                    if (container.height < img.height) {
                        
                        switch ($this.settings.verticalAlign.toLowerCase()) {
                            case 'middle':{
                                    overlapHeight = container.height - img.height;
                                    marginTop = overlapHeight / 2;
                                    $(image).css({
                                        'margin-top': marginTop + 'px'
                                    });
                                    break;
                            }
                            case 'bottom': {
                                    overlapHeight = container.height - img.height;
                                    marginTop = overlapHeight;
                                    $(image).css({
                                        'margin-top': marginTop + 'px'
                                    });
                                    break;
                            }
                            default:
                                break;
                        }
                    }
                }
            });
        },

        clearImageProps: function($el) {
            $el.find('img').css({
                'width': 'auto',
                'height': 'auto',
                'margin-left': '0',
                'margin-top': '0',
            });
        },

        fitImage: function(wrap) {
            var img = this.getImageSize(wrap);
            var container = this.getContainerSize(wrap);
            if (container.height > img.height) {
                $(this.settings.img).css({
                    'height': '100%',
                    'width': 'auto'
                });
                img = this.getImageSize(wrap);
                var overlapWidth, marginLeft;
                switch (this.settings.horizontalAlign.toLowerCase()) {
                    case 'center':{
                            overlapWidth = container.width - img.width;
                            marginLeft = overlapWidth / 2;
                            $(this.settings.img).css({
                                'margin-left': marginLeft + 'px'
                            });
                            break;
                    }
                    case 'right':{
                            overlapWidth = container.width - img.width;
                            marginLeft = overlapWidth;
                            $(this.settings.img).css({
                                'margin-left': marginLeft + 'px'
                            });
                            break;
                    }
                    default:
                        break;
                }
            } else {
                
                img = this.getImageSize(wrap);
                var overlapHeight, marginTop;
                if (container.height < img.height) {
                    
                    switch (this.settings.verticalAlign.toLowerCase()) {
                        case 'middle':{
                                overlapHeight = container.height - img.height;
                                marginTop = overlapHeight / 2;
                                $(this.settings.img).css({
                                    'margin-top': marginTop + 'px'
                                });
                                break;
                        }
                        case 'bottom':{
                                overlapHeight = container.height - img.height;
                                marginTop = overlapHeight;
                                $(this.settings.img).css({
                                    'margin-top': marginTop + 'px'
                                });
                                break;
                        }
                        default:
                            break;
                    }
                }
            }
        },

        setWidth: function($el) {
            $el.find('img').css({
                'width': '100%'
            });
        },

        getContainerSize: function($el) {
            var props = {};
            props.width = $($el).innerWidth();
            props.height = $($el).innerHeight();
            return props;
        },

        getContainerPadding: function($el) {
            var props = {};
            props.left = $el.css('padding-left').replace('px', '');
            props.right = $el.css('padding-right').replace('px', '');
            props.top = $el.css('padding-top').replace('px', '');
            props.bottom = $el.css('padding-bottom').replace('px', '');
            return props;
        },

        getImageSize: function($el) {
            var props = {};
            props.width = $($el).find('img').outerWidth();
            props.height = $($el).find('img').outerHeight();
            return props;
        },

    });

    
    $.fn[pluginName] = function(options) {
        
        var styles = ".ss-wrap-image .ss-image{  transition: all ease .1s; }";
        $('<style>' + styles + '</style>').appendTo(document.head);
        
        this.each(function() {
            if (!$.data(this, pluginName)) {
                $.data(this, pluginName, new Plugin(this, options));
            }
        });

        var resizeTimer;
        var resImg = new Plugin(null, options);
        $(window).on('resize', function(e) {
            clearTimeout(resizeTimer);
            resizeTimer = setTimeout(function() {
                resImg.fitImageOnResize($('.ss-wrap-image'));
            }, $.extend({}, defaults, options).resizeDelayTime);
        });

    };

}(jQuery));